#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define MAX_PREVOZNIK 31
#define MAX_STANJE 11

typedef struct autobusi_st {
    char prevoznik[MAX_PREVOZNIK];
    char stanje[MAX_STANJE];
    double cena_karte;
    unsigned broj_mesta;
    struct autobusi_st *sledeci;
} AUTOBUSI;

FILE *safe_fopen(char *naziv, char *rezim, int kod_greske);
void ucitaj_autobuse(FILE *ulazna, AUTOBUSI **pglava);
void ispisi_raspolozive_autobuse(FILE *izlazna, AUTOBUSI *glava);
double izracunaj_zaradu(AUTOBUSI *glava);
unsigned kapacitet_autoprevoznika(AUTOBUSI *glava, char prevoznik[]);

void inicijalizacija(AUTOBUSI **pglava);
AUTOBUSI *napravi_cvor(char prevoznik[],
                       char stanje[],
                       double cena_karte,
                       unsigned broj_mesta);
void dodaj_cvor(AUTOBUSI **pglava, AUTOBUSI *novi);
void obrisi_listu(AUTOBUSI **pglava);

int main(int argc, char **argv) {
    AUTOBUSI *glava;

    if(argc != 5) {
        printf("Primer poziva programa: %s Lasta autobusi.txt 138 obracun.txt\n", argv[0]);
        exit(EXIT_FAILURE);
    }

//    printf("Pre: %p\n", glava);
    inicijalizacija(&glava);
//    printf("Posle: %p\n", glava);

    FILE *ulazna = safe_fopen(argv[2], "r", 2);  
    ucitaj_autobuse(ulazna, &glava);
    fclose(ulazna);

    FILE *izlazna = safe_fopen(argv[4], "w", 3);
    fprintf(izlazna, "Raspolozivi ispravni autobusi:\n\n");
    ispisi_raspolozive_autobuse(izlazna, glava);

    unsigned kapacitet = kapacitet_autoprevoznika(glava, argv[1]);
    unsigned broj_putnika = atoi(argv[3]);
    fprintf(izlazna, "\n");
    if(kapacitet > broj_putnika) {
        fprintf(izlazna, "Prevoznik %s na letovanje moze da preveze sve prijavljene putnike.\n", argv[1]);
    } else {
        fprintf(izlazna, "Prevoznik %s ne moze da preveze sve prijavljene putnike na letovanje!\n", argv[1]);
    }

    double zarada = izracunaj_zaradu(glava);
    fprintf(izlazna, "Maksimalna moguca zarada svih autoprevoznika iznosi %.2lf dinara.\n", zarada);
    fclose(izlazna);

    obrisi_listu(&glava);  

    return EXIT_SUCCESS;
}

FILE *safe_fopen(char *naziv, char *rezim, int kod_greske) {
    FILE *fp = fopen(naziv, rezim);

    if(fp == NULL) {
        printf("Greska prilikom otvaranja fajla %s!\n", naziv);
        exit(kod_greske);
    }

    return fp;
}

void ucitaj_autobuse(FILE *ulazna, AUTOBUSI **pglava) {
    char tmp_prevoznik[MAX_PREVOZNIK];
    char tmp_stanje[MAX_STANJE];
    double tmp_cena_karte;
    unsigned tmp_broj_mesta;

    while(fscanf(ulazna, "%s %s %lf %u", 
                 tmp_prevoznik, tmp_stanje, &tmp_cena_karte, &tmp_broj_mesta) != EOF) {
        AUTOBUSI *novi = napravi_cvor(tmp_prevoznik, tmp_stanje,
                                      tmp_cena_karte, tmp_broj_mesta);
        dodaj_cvor(pglava, novi);
    }
}

void ispisi_raspolozive_autobuse(FILE *izlazna, AUTOBUSI *glava) {
    AUTOBUSI *tekuci = glava;

    while(tekuci != NULL) {
        if(strcmp(tekuci->stanje, "ispravan") == 0) {
            fprintf(izlazna, "%s %s %.2lf %u\n", tekuci->prevoznik, tekuci->stanje,
                                                 tekuci->cena_karte, tekuci->broj_mesta);
        }

        tekuci = tekuci->sledeci;
    }
}

double izracunaj_zaradu(AUTOBUSI *glava) {
    AUTOBUSI *tekuci = glava;
    double zarada = 0;

    while(tekuci != NULL) {
        zarada += tekuci->cena_karte * tekuci->broj_mesta;

        tekuci = tekuci->sledeci;
    }

    return zarada;
}

unsigned kapacitet_autoprevoznika(AUTOBUSI *glava, char prevoznik[]) {
    AUTOBUSI *tekuci = glava;
    unsigned kapacitet = 0;

    while(tekuci != NULL) {
        if(strcmp(tekuci->prevoznik, prevoznik) == 0 &&
           strcmp(tekuci->stanje, "ispravan") == 0) {
           kapacitet += tekuci->broj_mesta; 
        }

        tekuci = tekuci->sledeci;
    }

    return kapacitet;
}

void inicijalizacija(AUTOBUSI **pglava) {
    *pglava = NULL;
}

AUTOBUSI *napravi_cvor(char prevoznik[],
                       char stanje[],
                       double cena_karte,
                       unsigned broj_mesta) {
    AUTOBUSI *novi = (AUTOBUSI *)malloc(sizeof(AUTOBUSI));

    if(novi == NULL) {
        printf("Greska prilikom zauzimanja memorije!\n");
        exit(EXIT_FAILURE);
    }

//    (*novi).prevoznik
    strcpy(novi->prevoznik, prevoznik);
    strcpy(novi->stanje, stanje);
    novi->cena_karte = cena_karte;
    novi->broj_mesta = broj_mesta;
    novi->sledeci = NULL;

    return novi;
}

void dodaj_cvor(AUTOBUSI **pglava, AUTOBUSI *novi) {
    if(*pglava == NULL) {
        *pglava = novi;
    } else {
        AUTOBUSI *tekuci = *pglava;

        while(tekuci->sledeci != NULL) {
            tekuci = tekuci->sledeci;
        }

        tekuci->sledeci = novi;
    }
}

void obrisi_listu(AUTOBUSI **pglava) {
    AUTOBUSI *tmp;

    while(*pglava != NULL) {
        tmp = *pglava;
        *pglava = (*pglava)->sledeci;
        tmp->sledeci = NULL;
        free(tmp);
    }
}

